/*===================================================================
  Xwindow Realtime Simulator ver.2 (1994.8.7)
  by kitsune(originated by sasa (originated by  gutchi (originated in WTNB)))
===========================================================================*/

#ifndef XRS_H_INCLUDE
#define XRS_H_INCLUDE

//************************** include *********************************

#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>


//************************** define ***********************************

#define PI         M_PI
#define FONT       "6x13"  //text font

#define EVENTMAX     10     // event maximum number
#define PARAMAX      10     // parameter maximum number
#define WINMAX       10     // graphic window maximum number

#define WINDOWSIZEi   100       // default window size(x derection)
#define WINDOWSIZEj   100       // default window size(y derection)
#define LATTICESIZE     1       // default lattice unit size of window

#define COLORNUMBER   12        // default color number 
#define PHI       (-PI*2.0/5.0) // default color OFFSET phase 
#define ColorResource "/usr/local/lib/g++-include/XRS/XRS.color"
                               // default color resource name


//*************** struct & class & function declare **************************

class     XRS;

struct manip_i{               //struct for 1 argument(int) manipurator
          void  (*fp)(XRS&,int);
          int   arg;
          manip_i(void (*f)(XRS&,int),int a){fp=f;arg=a;}
};

struct manip_Pc{               //struct for 1 argument(char*) manipurator
          void  (*fp)(XRS&,char*);
          char*  arg;
          manip_Pc(void (*f)(XRS&,char*),char* a){fp=f;arg=a;}
};

struct manip_di{           //struct for 2 argument(double,int) manipurator
          void  (*fp)(XRS&,double,int);
          double x;
          int   arg;
          manip_di(void (*f)(XRS&,double,int),double y,int a){
                           fp=f;  x=y;  arg=a;}
};

struct manip_A_Uli{  //struct for 2 argument(unsigned long[],int) manipurator
          void  (*fp)(XRS&,unsigned long *,int);
          unsigned long *x;
          int   arg;
          manip_A_Uli(void (*f)(XRS&,unsigned long *,int)
		      ,unsigned long y[],int a){ fp=f;  x=y;  arg=a;}
};

struct manip_iii{          //struct for 3 argument(int,int,int) manipurator
          void  (*fp)(XRS&,int,int,int);
          int   arg1,arg2,arg3;
          manip_iii(void (*f)(XRS&,int,int,int),int a1,int a2,int a3){
                           fp=f;  arg1=a1;  arg2=a2; arg3=a3;}
};

struct manip_PFi_Uli{    //struct for 2 argument(unsigned long (*)(int),int)
          void  (*fp)(XRS&,unsigned long (*)(int), int);
          unsigned long (*func)(int);
          int   arg;
          manip_PFi_Uli(void (*f)(XRS&,unsigned long (*)(int),int),
              	  unsigned long (*g)(int),int a){fp=f; func=g; arg=a;}
};
                                                                   
inline double defaultdraw(double);
double** array(int,int);
void     delete_array(double **&,int=1);


//************************* class XRS ************************************

class XRS{

      int     winum,mapwinum,unmapwinum,funcnum,paranum,eventnum;
      int     *winevent;
	  int     eventmax,paramax,winmax;
      int     w,id,ke;
	  int     ww,en,pn;
      int     *ts; 

      double  *t;
      double  **p,*dp;
   
      void    (**eventfunc)(int),(**wineventfunc)(int);
      double  (**drawfunc)(double); 

      unsigned long  **color;
      unsigned long  **mapi;

      char    manip;
      char    **eventname,**paraname,**title;
      char    *space,*pause,*cont,*quit,*start;
      
    public:

      int      *i_max,*j_max,*colornum,*px;
      unsigned long  ***map;
      unsigned long  black,white;
      
      Window   root,cwin,*win;
      Display  *d;
      GC       gc,bgc,wgc,dgc,tgc;
      Colormap cm;
      XEvent   event;
  
     //------------- constractor & destractor ---------------------------
      
      XRS(int=WINMAX,int=PARAMAX,int=EVENTMAX);
      ~XRS(void);

      
     //------------- input operator << -----------------------------------

      XRS& operator<<(double&);                    //  definitions of    
      XRS& operator<<(char [126]);                  //     operator <<
      XRS& operator<<(void (*)(int));              //
      XRS& operator<<(double (*)(double));         //
      
      friend XRS& operator<<(XRS&,XRS& (*)(XRS &));//appliciator (no argment)
      friend XRS& operator<<(XRS&,manip_i);     //    (int)
	  friend XRS& operator<<(XRS&,manip_Pc);    //    (char*)
      friend XRS& operator<<(XRS&,manip_di);    //    (double,int)
      friend XRS& operator<<(XRS&,manip_A_Uli); //    (unsignted long[],int)
      friend XRS& operator<<(XRS&,manip_PFi_Uli);//(unsigned long(*)(int),int)
      friend XRS& operator<<(XRS&,manip_iii);   //    (int,int,int)
    									  
      friend XRS& PARAMETER(XRS&);                 // manipurator
      friend XRS& EVENT(XRS&);
      friend void setWINDOW(XRS&,int,int,int);
      friend void setUNMAP(XRS&,int);
      friend void setMONO(XRS&,double,int);
      friend void setPHASE(XRS&,double,int);
	  friend void setCOLOR(XRS&,char*);
	  friend void setCOLOR(XRS&,unsigned long[],int);
      friend void setCOLOR(XRS&,unsigned long (*)(int),int);

      
     //-------------- initialize function --------------------------------

      void  initialize(void);
      void  initialize(double &);
      void  initialize(int &);
      void  window_set(void);
      void  initialize_map(void);
      void  initialize_color(unsigned long[]);
      void  initialize_color(char,double);

      
     //--------------- event function ---------------------------------------
    
      int   checkevent(void);
      void  cwin_event(void);                    // control window
      void  win_event(int);                      // graphic windows


     //--------------- drawing a window -----------------------------------

      void  cwin_draw(void);                     // control window
      void  current_parameter(void);             //   writing parameter values   
      
      void  draw_2D(double**,int=0);         // drawing 2D graphics
      void  draw_point(double,int,int,int=0);// drawing a point
      void  clear_window(int=0);             // graphic window clear

     //------------ string format function --------------------------------

     void string(char*,char*,int);
     void parameter_string(char*,char*,double,double);
     
     //------------ setting a graphic window title -----------------------

     void set_title(char*,int=0);

     //------------ getting a color pixel & a graphic context-------------

     unsigned long  color_pixel(double,double,double=1.0);
     unsigned long  color_pixel(char*);
	 int color_pixel(unsigned long *&,char* =ColorResource);
	 
     GC graphic_context(unsigned long,unsigned int=0,int=LineSolid,
           int=CapButt,int=JoinMiter,char* =FONT);
	 int graphic_context(GC *&,char* =ColorResource);

	 //------------ freind function ----------------------
	 friend void sample_window(XRS &,int=0);
};

//************************** manipurator ***********************************

XRS& operator<<(XRS&,XRS& (*)(XRS &));
XRS& operator<<(XRS&,manip_i);
XRS& operator<<(XRS&,manip_Pc);
XRS& operator<<(XRS&,manip_di);
XRS& operator<<(XRS&,manip_A_Uli);
XRS& operator<<(XRS&,manip_PFi_Uli);
XRS& operator<<(XRS&,manip_iii);

XRS& PARAMETER(XRS&);
XRS& EVENT(XRS&);

void setWINDOW(XRS&,int,int,int);
manip_iii WINDOW(int=WINDOWSIZEi,int=WINDOWSIZEj,int=LATTICESIZE);

void setUNMAP(XRS&,int);
manip_i UNMAP(int=0);

void setMONO(XRS&,double,int);
manip_di MONO(double=PHI,int=COLORNUMBER);

void setPHASE(XRS&,double,int);
manip_di PHASE(double=PHI,int=COLORNUMBER);

void setCOLOR(XRS&,char*);
manip_Pc COLOR(char* =ColorResource);

void setCOLOR(XRS&,unsigned long[],int);
manip_A_Uli COLOR(unsigned long[],int=COLORNUMBER);

void setCOLOR(XRS&,unsigned long (*)(int),int);
manip_PFi_Uli COLOR(unsigned long (*)(int),int=COLORNUMBER);
			
#undef PI
#undef FONT
#undef EVENTMAX
#undef PARAMAX
#undef WINMAX
#undef WINDOWSIZEi
#undef WINDOWSIZEj
#undef LATTICESIZE
#undef COLORNUMBER
#undef PHI
#undef ColorResource

#endif










