/*===================================================================
  Xwindow Realtime Simulator ver.2 (1995.12.12)
  by kitsune(originated by sasa (originated by  gutchi (originated in WTNB)))
===========================================================================*/

//************************** include *********************************

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <XRS/XRS.h>


//************************** define ***********************************

//--------- constant ( must not be changed) ---------

#define PI        M_PI
#define MCOLOR    65535
#define PHI_RED   (PI/3.0)       // RED phase                                  
#define PHI_GREEN (PI)           // GREEN phase                                
#define PHI_BLUE  (-PI/3.0)      // BLUE phase                                 


//-------- constant ( may be changed) ---------

#define FXNUM       7      // character number of event & parameter name
#define FXNUMD    "%7.7lf" // ="% FXNUM.* lf" printf format of double 
#define FXNUMI    "%7d"    // ="% FXNUM d"    printf format of int
#define FX     (7*FXNUM+1) // column width of control window
#define FY          20     // row height of control window 
#define TITLENUM    16     // character number of graphic window title

#define OWINTITLE  "Control Window" //control window title
#define PARAMCOLOR  "blue"  // color of parameter change indicator
#define DPARAM          1   // initial value of parameter change unit 
#define AMPF          0.8   // default amplitude in defaultdraw function


//-------- default value ( can be changed in your program ) -----------

#define WINDOWSIZEi   100       // default window size(x derection)
#define WINDOWSIZEj   100       // default window size(y derection)
#define LATTICESIZE     1       // default lattice unit size of window

#define COLORSTYLE    'M'       // default color style 'M'(mono) or 'P'(phase)
#define COLORNUMBER   12        // default color number 
#define PHI       (-PI*2.0/5.0) // default color OFFSET phase 



//*********************** string format function ***********************

void XRS::string(char *p,char* s,int n)
{
    register i,j,k;
    char ss[64];

    i=strlen(s);
    if( i > n ) i=n;
    k=(int)((n-i)/2);
    for( j=0; j<k; j++ ) ss[j]=' ';
    ss[k]='\0';
    strncat(ss,s,i);
    for(j=k+i; j<n; j++) ss[j]=' ';
    ss[n]='\0';
    
    strcpy(p,ss);
}


void XRS::parameter_string(char *s,char *ds,double p,double dp)
{
   register int k,n,i,digit;
   char *dot;

   digit=(int)floor(log10(dp)+0.5);
   if(digit>=0) digit++;

   sprintf(s,"%7.7lf",p);
   i=strlen(s);
   for(k=0; k<i; k++) ds[k]=' ';
   ds[i]='\0';

   if( (dot=strrchr(s,'.'))==NULL ) dot=strrchr(s,'\0');
   n=dot-digit-s;
   if( n >=0 && n < i) ds[n]=*(dot-digit);
}



//****************** window title *************************************

void XRS::set_title(char *name,int w)
{
     string(title[w],name,TITLENUM);
     XStoreName(d, win[w], title[w]);
}



//******************* constractor & destructor **************************

XRS::XRS(int wmax,int pmax,int emax)
{
      register int k;

      winmax=wmax;
      eventmax=emax;
      paramax=pmax;

//--------- display -------------------
      if((d=XOpenDisplay(NULL))==NULL){
          fprintf(stderr,"can't open DISPLAY."); exit(1);   }
      root = RootWindow(d, 0);
      cm = DefaultColormap(d, 0);

      black = BlackPixel(d, 0); 
      white = WhitePixel(d, 0);

      gc = graphic_context(black,1);
      bgc = graphic_context(black,0);
      wgc = graphic_context(white,0);
      dgc = graphic_context(black,0,LineOnOffDash);
      tgc = graphic_context(color_pixel(PARAMCOLOR),0);

//-------- PARAMETER ---------------
      pn=0;
      paranum=0;
      p        = new double*[paramax];
      dp       = new double [paramax];
      paraname = new char*  [paramax];
      for(k=0;k<paramax;k++){
          paraname[k] = new char[FXNUM+1];
          dp[k]=DPARAM;
      }

//-------- EVENT --------------
      en=0;
      eventnum=0;
      eventname = new     char*     [eventmax];
      eventfunc = new (void(*)(int))[eventmax];
      for(k=0;k<eventmax;k++)
          eventname[k] = new char[FXNUM+1];

      space = new char[FXNUM+1];
      pause = new char[FXNUM+1];
      cont  = new char[FXNUM+1];
      quit  = new char[FXNUM+1];
      start = new char[FXNUM+1];

//-------- WINDOW ---------------
      
      ww=0;
      winevent     = new int                [winmax];
      wineventfunc = new (void(*)(int))     [winmax];
      drawfunc     = new (double(*)(double))[winmax];
      color        = new unsigned long*     [winmax];
      mapi         = new unsigned long*     [winmax];
      title        = new char*              [winmax];
      i_max        = new int                [winmax];
      j_max        = new int                [winmax];
      colornum     = new int                [winmax];
      px           = new int                [winmax];
      map          = new unsigned long **   [winmax];
      win          = new Window             [winmax];

      winum=0;
      unmapwinum=0;
      char wanya[TITLENUM+1];
      for(w=0;w<winmax;w++){
          title[w]=new char[TITLENUM+1];
          sprintf(wanya,"window %d",w);
          string(title[w],wanya,TITLENUM);
          i_max[w]=WINDOWSIZEi;
          j_max[w]=WINDOWSIZEj;
          px[w]=LATTICESIZE;
          winevent[w]=0;
          colornum[w]=COLORNUMBER;
          initialize_color(COLORSTYLE,PHI);
          drawfunc[w]=defaultdraw;
      } 
}


XRS::~XRS(void)
{
    register int k;

//------ PARAMETER -----------
    for(k=0; k<paramax;k++)
        delete [] paraname[k];
    delete [] p;
    delete [] dp;
    delete [] paraname;

//------ EVENT ---------------
    for(k=0; k<eventmax;k++)
        delete [] eventname[k];
    delete [] eventname;
    delete [] eventfunc;

    delete [] space;
    delete [] pause;
    delete [] cont;
    delete [] quit;
    delete [] start;

//------ WINDOW --------------
    for(k=0; k<mapwinum; k++){
        delete [] title[k];
        delete [] color[k];
        delete [] mapi[k];
        delete [] map[k];
    }

    delete [] winevent;
    delete [] wineventfunc;
    delete [] drawfunc;
    delete [] color;
    delete [] mapi;
    delete [] title;
    delete [] i_max;
    delete [] j_max;
    delete [] colornum;
    delete [] px;
    delete [] map;
    delete [] win;

//------- display -----------
    XCloseDisplay(d);
}


//************************** manipurator ***********************************

XRS& operator<<(XRS& s,XRS& (*f)(XRS &)){return f(s);}
XRS& operator<<(XRS& s,manip_i m)    {m.fp(s,m.arg); return s;}
XRS& operator<<(XRS& s,manip_Pc m)   {m.fp(s,m.arg); return s;}
XRS& operator<<(XRS& s,manip_di m)   {m.fp(s,m.x,m.arg); return s;}
XRS& operator<<(XRS& s,manip_A_Uli m){m.fp(s,m.x,m.arg); return s;}
XRS& operator<<(XRS& s,manip_PFi_Uli m){m.fp(s,m.func,m.arg); return s;}
XRS& operator<<(XRS& s,manip_iii m)  {m.fp(s,m.arg1,m.arg2,m.arg3); return s;}

XRS& PARAMETER(XRS& s){ s.manip='P'; return s;}
XRS& EVENT(XRS& s){ s.manip='E'; return s;}

void setWINDOW(XRS& s,int i,int j,int k)
{
    if(s.ww<s.winmax){
         s.manip='W';   s.winum=s.ww+1;
         s.w=s.ww; s.i_max[s.ww]=i;  s.j_max[s.ww]=j; s.px[s.ww]=k;
         s.ww++;
    }else{
    	fprintf(stderr,"too many window!! Redefine winmax.\n"); exit(1);
    }
    
}
manip_iii WINDOW(int i,int j,int k)
{ return manip_iii(setWINDOW,i,j,k); }

void setUNMAP(XRS& s,int n)
{    s.unmapwinum=n; }
manip_i UNMAP(int n){return manip_i(setUNMAP,n);}

void setMONO(XRS& s,double x,int n)
{    delete [] s.color[s.w]; s.colornum[s.w]=n; s.initialize_color('M',x); }
manip_di MONO(double x,int n)
{    return manip_di(setMONO,x,n); } 

void setPHASE(XRS& s,double x,int n)
{    delete [] s.color[s.w]; s.colornum[s.w]=n; s.initialize_color('P',x); }
manip_di PHASE(double x,int n)
{    return manip_di(setPHASE,x,n); }

void setCOLOR(XRS& s,char* color_resource)
{    
     unsigned long *c;
     delete [] s.color[s.w]; 
     s.colornum[s.w]=s.color_pixel(c,color_resource);
     s.initialize_color(c); 
     delete [] c;
}
manip_Pc COLOR(char* c)
{    return manip_Pc(setCOLOR,c); }

void setCOLOR(XRS& s,unsigned long c[],int n)
{    delete [] s.color[s.w]; s.colornum[s.w]=n; s.initialize_color(c); }
manip_A_Uli COLOR(unsigned long c[],int n)
{    return manip_A_Uli(setCOLOR,c,n); }

void setCOLOR(XRS& s,unsigned long (*g)(int),int n)
{
     unsigned long *c=new unsigned long [n];
     delete [] s.color[s.w]; s.colornum[s.w]=n;
     for(register int i=0; i<n; i++) c[i]=g(i);
     s.initialize_color(c); 
     delete [] c;
}
manip_PFi_Uli COLOR(unsigned long (*g)(int),int n)
{    return manip_PFi_Uli(setCOLOR,g,n); }


//********************** definition of operator << **************************


XRS& XRS::operator<<(double& s)  
{
      switch(manip){
         case  'P' :             // pointers of parameter 
            if(paranum<paramax){
    			p[paranum]=&s; ++paranum;
    		}else{
    			fprintf(stderr,"too many parameter!! Redefine paramax.\n"); exit(1);
    		}
            break;
         default   :;
      }
      return *this;
}


XRS& XRS::operator<<(char name[64]) 
{
      switch(manip){
         case  'P' :              // parameter name 
            if(pn<paramax){ string(paraname[pn],name,FXNUM); ++pn; }
            break;
         case  'W' :              // graphic window title
            if(w<winmax) string(title[w],name,TITLENUM);
            break;
         case  'E' :              // event name
            if(en<eventmax){ string(eventname[en],name,FXNUM); ++en; }
            break;
         default   :;
      }
      return *this;
}


XRS& XRS::operator<<(void (*f)(int))
{
      switch(manip){
         case  'W' :
            wineventfunc[w]=f;     // graphic window event function
        	winevent[w]=1;
        	break;			
         case  'E' :               // control window event function
            if(eventnum<eventmax){
    			eventfunc[eventnum]=f;  ++eventnum;
    		}else{
    			fprintf(stderr,"too many event!! Redefine eventmax.\n"); exit(1);
    		}
            break;
         default   :;
      }
      return *this;
}
      

XRS& XRS::operator<<(double (*f)(double))
{
      switch(manip){
         case  'W' :                // drawing function
            drawfunc[w]=f;          
            break;
         default   :;
      }
      return *this;
}



//***************** window_setting & initialize ****************************

void  XRS::initialize(void)        {           id=0;   window_set();}
void  XRS::initialize(double &time){ t=&time;  id=-1;  window_set();}
void  XRS::initialize(int &step)   { ts=&step; id=1;   window_set();}


void XRS::window_set(void)
{
//------- graphic window set -------

      for(int k=0;k<winum;k++){
             win[k] = XCreateSimpleWindow(d, root, 300, 300,i_max[k]*px[k]
                               ,j_max[k]*px[k], 2, black, white);
             XStoreName(d, win[k], title[k]);
             XSelectInput(d, win[k] , ButtonPressMask|ExposureMask );
             XMapRaised(d, win[k]);
      }

      mapwinum=winum-unmapwinum; if(mapwinum<0) mapwinum=0;
      initialize_map();

//------- control window set-----------

      if(id){
          funcnum=2+1+eventnum+(1+eventnum)%2+2*paranum;
          ke=1+(1+eventnum+(1+eventnum)%2)/2;
      }else{
          funcnum=2+eventnum+eventnum%2+2*paranum;
          ke=1+(eventnum+eventnum%2)/2;
      }
     
      cwin = XCreateSimpleWindow(d, root, 100, 50, FX*(funcnum/2)+1,
                       FY*2+1,2, black, white);
      XStoreName(d, cwin, OWINTITLE);
      XSelectInput(d, cwin, StructureNotifyMask);
      XMapRaised(d, cwin);

//------- window initialization -----

      string(space,""     ,FXNUM);
      string(start,"START",FXNUM);
      string(quit ,"QUIT" ,FXNUM);
      string(pause,"PAUSE",FXNUM);
      string(cont ,"CONT" ,FXNUM);

      while(event.type != MapNotify)
          XWindowEvent(d, cwin, StructureNotifyMask, &event);

      XSelectInput(d, cwin,ButtonPressMask|ExposureMask);
      cwin_draw();

      XDrawImageString(d, cwin, bgc,    0+5, FY-4, start, FXNUM);

      XFlush(d);
   
      XNextEvent(d, &event);
      while(event.type != ButtonPress)
          XNextEvent(d, &event);
      XDrawImageString(d, cwin, bgc,    0+5, FY-4, quit, FXNUM);
}


//********************* initialize & delete map **************************

void  XRS::initialize_map()
{
      register int i,j,w;

      for(w=0; w<mapwinum; w++){
          mapi[w] = new unsigned long  [i_max[w]*j_max[w]];
          map[w]  = new unsigned long* [i_max[w]];
          for(i=0; i<i_max[w]; i++) map[w][i]=mapi[w]+i*j_max[w];
      
          for(j=0; j<j_max[w]; j++)
              for(i=0; i<i_max[w]; i++)
                  map[w][i][j] = white;
      }
}


//******************** initialize color ************************

void  XRS::initialize_color(unsigned long c[])
{
      register int i;

      color[w]=new unsigned long[colornum[w]+2];
      for(i=0; i<colornum[w]; i++)  color[w][i]=c[i];

      color[w][colornum[w]] = black;
      color[w][colornum[w]+1] = white;
}


void  XRS::initialize_color(char s,double phi)
{
      register int i;

      color[w]=new unsigned long[colornum[w]+2];
      for(i=0; i<colornum[w]; i++){
         if(s=='M') color[w][i]=color_pixel((double)(i)/colornum[w],phi,1);
         if(s=='P') color[w][i]=color_pixel(1,phi+2*PI*(i+0.5)/colornum[w],1);
      }

      color[w][colornum[w]] = black;
      color[w][colornum[w]+1] = white;
}


//************************* event *******************************

int XRS::checkevent(void)
{
      register int k,re;
      char     wanya[64];

      re=0;
      if(XCheckWindowEvent(d, cwin, ButtonPress|ExposureMask , &event))
          { cwin_event(); re|=1; }
      
      for(k=0;k<winum;k++)
          if(XCheckWindowEvent(d, win[k], ButtonPress|ExposureMask , &event))
             { win_event(k); re|=2; re|=k<<2; }

      if(id){
         XDrawImageString(d, cwin, bgc, FX+5, FY-4,space, FXNUM);
         if(id==-1) sprintf(wanya, FXNUMD,*t);
         else      sprintf(wanya, FXNUMI,*ts);     
         XDrawImageString(d, cwin, bgc, FX+5, FY-4, wanya, FXNUM);
      }
      return re;
}


//*********************** control window event ***************************** 

void  XRS::cwin_event(void)
{
      if( event.type == Expose ){ cwin_draw(); return;}

      int fun,xfun,yfun,funid;
          
      xfun=event.xbutton.x/FX;
      yfun=event.xbutton.y/FY;
      fun=2*xfun+yfun;
      if(id) funid=1; else funid=0;
      
      if (fun == 0){
          /* Quit */
          exit(1);
      
      } else if (fun == 1){
          /* Pause/Cont */
          cwin_draw();
          XDrawImageString(d, cwin, bgc,   5,   2*FY-4,cont, FXNUM);
          XWindowEvent(d, cwin, ButtonPress,  &event);
          XDrawImageString(d, cwin, bgc,   5,   2*FY-4,pause, FXNUM);

      } else if (fun == 1+funid){
          /* Time or Step value */
          if(id==-1)     *t=0;
          else if(id==1)  *ts=0;

      } else if (fun>=2+funid && fun<eventnum+2+funid ){
          /* each eventfunction */
          eventfunc[fun-2-funid](event.xbutton.button);  
          current_parameter();

      } else if (fun >= 2*ke && fun < funcnum){
          /* parameter change */
          
          if (yfun == 0){
              /* parameter up */
              if (event.xbutton.button==1)
                  *p[xfun-ke]+=dp[xfun-ke];
              else if (event.xbutton.button==2)
                  *p[xfun-ke]+=dp[xfun-ke]*0.1;
              else if (event.xbutton.button==3)
                  dp[xfun-ke]=dp[xfun-ke]*10.0;
              current_parameter();
 
          }else{
              /* parameter down */
              if (event.xbutton.button==1)
                  *p[xfun-ke]-=dp[xfun-ke];
              else if (event.xbutton.button==2)
                  *p[xfun-ke]-=dp[xfun-ke]*0.1;
              else if (event.xbutton.button==3)
                  dp[xfun-ke]=dp[xfun-ke]*0.1;
              current_parameter();
 
          }
     
      } else {
      }
}
         

//****************** graphic window event ************************

void  XRS::win_event(int k)
{
    if( k<0 || k>=winum ){ 
       fprintf(stderr,"In XRS::win_event(int w), w is not correct.\n"); exit(1);}  

    if(  event.type == Expose ){
        register i,j,ip,jp;
        if( k<mapwinum )
            for(i=0; i<i_max[k]; i++)
                for(j=0; j<j_max[k]; j++){
                    XSetForeground(d, gc, map[k][i][j]);
                    for(ip=0; ip<px[k]; ip++)
                       for(jp=0; jp<px[k]; jp++)
                           XDrawPoint(d, win[k], gc, i*px[k]+ip, j*px[k]+jp);
                }
        return;
    }

    if( winevent[k] ) wineventfunc[k](event.xbutton.button);
}



//******************** control window draw **************************

void  XRS::cwin_draw(void)
{
      register int i,j,k,funid;
      char wanya[64],dwanya[64];

      XDrawLine(d, cwin, bgc, 0, FY, FX*(funcnum+1)/2+1, FY);
      for (i=1;i<funcnum/2;i++){
           XDrawLine(d, cwin, bgc, FX*i, 0, FX*i, FY*2);
      }

      XDrawImageString(d, cwin, bgc,    0+5,   FY-4,quit, FXNUM);
      XDrawImageString(d, cwin, bgc,    0+5, 2*FY-4,pause, FXNUM);

      if(id) funid=1; else funid=0;
      for (i=0; i<eventnum; ++i){ 
          j=(i+funid)%2+1; k=1+((i+funid)-(i+funid)%2)/2;
          XDrawImageString(d, cwin, bgc, k*FX+5, j*FY-4, eventname[i], FXNUM);
      }

      for(i = 0; i<paranum; ++i){
          k=i+ke;
          XDrawImageString(d, cwin, bgc, k*FX+5,   FY-4,  paraname[i], FXNUM);
          parameter_string(wanya,dwanya,*p[i],dp[i]);
          XDrawImageString(d, cwin, bgc, k*FX+5, 2*FY-4,  wanya, FXNUM);
          XDrawString(d, cwin, tgc, k*FX+5, 2*FY-4, dwanya, FXNUM);
      }
}


//************* writing current parameters in a control window ***************

void  XRS::current_parameter(void)
{
    char wanya[64],dwanya[64];
    for(int i=0; i<paranum; ++i){
        int k=i+ke; 
        XDrawImageString(d, cwin, bgc, k*FX+5, 2*FY-4,space, FXNUM);
        parameter_string(wanya,dwanya,*p[i],dp[i]);
        XDrawImageString(d, cwin, bgc, k*FX+5, 2*FY-4,  wanya, FXNUM);
        XDrawString(d, cwin, tgc, k*FX+5, 2*FY-4, dwanya, FXNUM);
    }
}


//************* 2D graphics  ********************************

void  XRS::draw_2D(double **W,int w)
{
      register i,j,ip,jp,digA,N,imax,jmax,pxw;
      unsigned long  newmap;
      Window   winw;
      double   (*f)(double);
      unsigned long **Mw;

      if( w<0 || w>=winum ){
       fprintf(stderr,"In XRS::draw_2D(double **W,int w), w is not correct.\n"); exit(1);}
      
      imax=i_max[w];
      jmax=j_max[w];
      pxw=px[w];
      winw=win[w];
      N=colornum[w];
      f=drawfunc[w];
      Mw=map[w];
      
      for(i=0; i<imax; i++){
          for(j=0; j<jmax; j++){
              digA=(int)(f(W[i][j])*N);  
              if(digA > N) digA=N; if(digA < 0) digA=N+1;
              newmap = color[w][digA];

              if(w<mapwinum){
                   if(newmap == Mw[i][j]) continue;
                   Mw[i][j] = newmap;
              }

              XSetForeground(d, gc, newmap);
              for(ip=0; ip<pxw; ip++)
                   for(jp=0; jp<pxw; jp++)
                       XDrawPoint(d, winw, gc, i*pxw+ip, j*pxw+jp);
          }
      }
}

      

//************* drawing a point in graphic windows****************************

void  XRS::draw_point(double W,int i,int j,int w)
{

      if( w<0 || w>=winum ){
       fprintf(stderr,"In XRS::draw_point(double W,int i,int j,int w), w is not correct.\n"); exit(1);}

      if( i<0 || i>=i_max[w] || j<0 || j>=j_max[w]) return; 
      
      register ip,jp,digA;
      unsigned long  newmap;

      digA=(int)(drawfunc[w](W)*colornum[w]);
      if(digA > colornum[w]) digA=colornum[w];
      if(digA < 0) digA=colornum[w]+1;
      newmap = color[w][digA];

      if(w<mapwinum){
          if(newmap == map[w][i][j]) return;
          map[w][i][j] = newmap;
      }

      XSetForeground(d, gc, newmap);
      for(ip=0; ip<px[w]; ip++)
          for(jp=0; jp<px[w]; jp++)
              XDrawPoint(d, win[w], gc, i*px[w]+ip, j*px[w]+jp);
      
}

//************* graphic window clear ****************************

void  XRS::clear_window(int w)
{
     if( w<0 || w>=winum ){
       fprintf(stderr,"In XRS::clear_window(int w), w is not correct.\n"); exit(1);}

      register int i,j;
      XClearWindow(d,win[w]);
      if(w<mapwinum){
          for(j=0; j<j_max[w]; j++)
    	      for(i=0; i<i_max[w]; i++)  map[w][i][j] = white;
      }
}


//************* color pixel ****************************************

unsigned long XRS::color_pixel(double r,double phi,double brightness)
{
      double odias, dc, max;
      XColor   cc;

      max=brightness*MCOLOR;
      if( max < 0      ) max=0;
      if( max > MCOLOR ) max=MCOLOR;
      if(   r < 0      ) r=0;
      if(   r > 1      ) r=1;
      
      odias = 1.0-r;
      
      dc=cos(phi-PHI_RED);
      if(dc > 0.5)
         cc.red = (int)max;
      else if (dc < -0.5)
         cc.red = (int)(max*odias);
      else
         cc.red = (int)(max*(odias+(1.0-odias)*(dc+0.5)));

      dc=cos(phi-PHI_GREEN);
      if(dc > 0.5)
         cc.green = (int)max;
      else if (dc < -0.5)
         cc.green = (int)(max*odias);
      else
         cc.green = (int)(max*(odias+(1.0-odias)*(dc+0.5)));

      dc=cos(phi-PHI_BLUE);
      if(dc > 0.5)
         cc.blue = (int)max;
      else if (dc < -0.5)
         cc.blue = (int)(max*odias);
      else
         cc.blue = (int)(max*(odias+(1.0-odias)*(dc+0.5)));

      XAllocColor(d, cm, &cc);
      return cc.pixel;
}


unsigned long XRS::color_pixel(char* name)
{
      XColor   cc;

      if(!XParseColor(d,cm,name,&cc)){
          printf("Color request fails. No color has a such name\n");
          exit(1);
      }

      XAllocColor(d, cm, &cc);
      return cc.pixel;
}


int XRS::color_pixel( unsigned long *&color,char *color_resource)
{
      register int n,i;
      char buffer[256],*colorname;
      unsigned long cp[256];
      FILE *file;

      if((file=fopen(color_resource,"r"))==NULL){
          printf("%s can't open.\n",color_resource);
          exit(1);
      }

      for(n=0;n<256;n++){
          if(fgets(buffer,sizeof(buffer),file)==NULL){ break;}
          /*--- remove spaces on top ----*/
          i=0; while(buffer [i]==' '){ ++i; } colorname=buffer+i;
          /*--- skip a empty line and a comment line ----*/
          if(colorname[0]=='\n' || colorname[0]=='!'){ n--; continue; };
          /*--- remove spaces at end -----*/
          i=strlen(colorname)-1; if(colorname[i]=='\n'){ colorname[i]=' '; i--;} 
          while(colorname[i]==' '){ --i; } colorname[i+1]='\0';

          cp[n]=color_pixel(colorname);
      }

      color=new unsigned long [n];
      for(i=0; i<n; i++) color[i]=cp[i];

      fclose(file);
      return n;
}


//*******************  graphic context ********************
   
GC XRS::graphic_context(unsigned long color,unsigned int line_width,
           int line_style,int cap_style,int join_style,char* font)
{
     GC g;
     Font dfont;
     dfont = XLoadFont(d, font);     
     g = XCreateGC(d, root, 0, 0);
     XSetLineAttributes(d, g, line_width, line_style, cap_style, join_style);
     XSetForeground(d, g, color);
     XSetBackground(d, g, white);
     XSetFont(d, g, dfont);
     return g;
}


int XRS::graphic_context(GC *&g,char *color_resource)
{
      register int i,n;
      unsigned long *c;

      n=color_pixel(c,color_resource);
      g=new GC [n];
      for(i=0; i<n; i++)  g[i]=graphic_context(c[i]);

      delete [] c;
      return n;
}


//********** default draw function ********************************

double defaultdraw(double x){ return(x*x*AMPF);}


//********** sample window draw function **************************

void sample_window(XRS &s,int w)
{
     if( w<0 || w>=s.winum ){
       fprintf(stderr,"In sample_window(XRS &s,int w), w is not correct.\n"); exit(1);}

     register i,j,imax,jmax,m,sj;
     double mono,sashi,ss;

     imax=s.i_max[w];
     jmax=s.j_max[w];

     for( j=0; j<jmax; j++)
         for( i=0; i<imax; i++) s.draw_point((double)i/imax,i,j,w);

     mono=(double)imax*s.px[w]/2;
     sashi=mono/5;

     for(j=1; j<10; j++){
         ss=j*sashi;
         sj=(int)(floor(ss)+floor(2*(ss-floor(ss))));
         XDrawLine(s.d,s.win[w],s.bgc,sj,0,sj,5);
     }
     
     m=(int)(floor(mono)+floor(2*(mono-floor(mono))));
     XDrawLine(s.d,s.win[w],s.bgc,m,0,m,10);
}


//*************** making a 2D array ********************************
   
double** array(int imax,int jmax){
    register int i;
    double *Mi,**M;

    Mi=new double[imax*jmax];
    M=new double*[imax];
    for(i=0; i<imax; i++) M[i]=Mi+i*jmax;
    return M;
}

void delete_array(double **&M,int imax){
    register int i;
    delete [] M[0]; for(i=0; i<imax; i++) M[i]=NULL;
    delete [] M; M=NULL;
}
    		
#undef PI
#undef MCOLOR
#undef PHI_RED
#undef PHI_GREEN
#undef PHI_BLUE
#undef FX
#undef FY
#undef FXNUM
#undef FXNUMD
#undef FXNUMI
#undef TITLENUM
#undef OWINTITLE
#undef PARAMCOLOR
#undef DPARAM
#undef AMPF
#undef WINDOWSIZEi
#undef WINDOWSIZEj
#undef LATTICESIZE
#undef COLORSTYLE
#undef COLORNUMBER
#undef PHI











