/*********** include **************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>


#define pi        3.14159265358979
#define MCOLOR    65535
#define PHI_RED   (pi/3.0) 
#define PHI_GREEN (pi)     
#define PHI_BLUE  (-pi/3.0)


Display  *d;
GC       gc,bgc;
Colormap cm;
Window   root,win;
XEvent    event;

void help(){
printf(" This command shows a sample color disk of given brightness\n"); 
printf("according to color pixels which XRS::color_pixel(r,,brightness)\n");
printf("returns. ( 0<= r, brightness <= 1 )\n");
printf("\n");
printf(" how to use: colorsample [(float)brightness] [(int)windowsize]\n");
printf(" !This command consumes many colorcells of a workstation.\n");
exit(1);
}
            
void colormap(int,float);
unsigned long color_pixel(float,float,float);


/********** main ***********/

main(int argc,char *argv[])
{
    float bright;
    int    wide;
    unsigned long black,white;

    if( argc<=1 || argc >3 ) help();
    bright=atof(argv[1]);
	if( argc==3 )  wide=atoi(argv[2]);  else  wide=100;
	if( bright<0 || bright >1 ) help();

    d = XOpenDisplay(NULL);
    root = RootWindow(d, 0); 

	cm=DefaultColormap(d, 0);
    black = BlackPixel(d, 0);
	white = WhitePixel(d, 0);
    gc = XCreateGC(d, root, 0, 0);
    bgc = XCreateGC(d, root, 0, 0);

    XSetLineAttributes(d, gc, 1, LineSolid, CapButt, JoinMiter);
    XSetLineAttributes(d, bgc, 0, LineSolid, CapButt, JoinMiter);

    XSetForeground(d, bgc, black);
    XSetBackground(d, bgc, white);

    win= XCreateSimpleWindow(d, root,300,300,wide,wide,2,black,white);
    XStoreName(d, win, "Color Sample");
    XSelectInput(d, win,StructureNotifyMask);
    XMapRaised(d, win);

    while(event.type != MapNotify)
        XWindowEvent(d, win, StructureNotifyMask, &event);

    XSelectInput(d, win,ButtonPressMask);
    XFlush(d);

    colormap(wide,bright);

    XNextEvent(d, &event);
    while(event.type != ButtonPress)
       XNextEvent(d, &event);

    XCloseDisplay(d);                 
}



void colormap(int wide,float b)
{
   int i,j,size,center,radius;
   float r,theta;
   double x,y,s;
   unsigned long map;

   if(wide<20) return;
   size=wide-10;
   center=(int)((float)wide/2);
   radius=(int)((float)size/2);

   for(j=-center; j<center; j++){
      y=(double)j/radius;
      for(i=-center; i<center; i++){
          x=(double)i/radius;
          r=sqrt(x*x+y*y);
          theta=atan2(y,x);  
          if(r>=1.0) continue;
          r=(float)(int)(r*5)/5;
          theta=(float)(2*pi*(int)(theta*20/(2*pi))/20);
          map=color_pixel(r,theta,b);
          XSetForeground(d, gc, map);
          XDrawPoint(d, win, gc,i+center,center-j);
      }
   }

   XDrawArc(d,win,bgc,
       center-radius,center-radius,2*radius,2*radius,0,360*64);
   XDrawLine(d,win,bgc,center+radius-10,center,center+radius,center);
}


unsigned long color_pixel(float r,float phi,float brightness)
{
      float odias, dc, max;
      XColor   cc;

      max=brightness*MCOLOR;
          if( max < 0      ) max=0;
          if( max > MCOLOR ) max=MCOLOR;
          if(   r < 0      ) r=0;
          if(   r > 1      ) r=1;
          
      odias = 1.0-r;
      
      dc=cos(phi-PHI_RED);
      if(dc > 0.5)
         cc.red = (int)max;
      else if (dc < -0.5)
         cc.red = (int)(max*odias);
      else
         cc.red = (int)(max*(odias+(1.0-odias)*(dc+0.5)));

      dc=cos(phi-PHI_GREEN);
      if(dc > 0.5)
         cc.green = (int)max;
      else if (dc < -0.5)
         cc.green = (int)(max*odias);
      else
         cc.green = (int)(max*(odias+(1.0-odias)*(dc+0.5)));

      dc=cos(phi-PHI_BLUE);
      if(dc > 0.5)
         cc.blue = (int)max;
      else if (dc < -0.5)
         cc.blue = (int)(max*odias);
      else
         cc.blue = (int)(max*(odias+(1.0-odias)*(dc+0.5)));

      XAllocColor(d, cm, &cc);
      return cc.pixel;
}

